﻿Imports Microsoft.AspNet.Identity
Imports System.Collections.Generic
Imports System.Linq
Imports System.Threading.Tasks

Namespace ASPNETIdentityTest
    Public Class AppUser
        Implements IUser

        Private m_Id As String
        Private m_Password As String

        Public Property UserId As String
            Get
                Return m_Id
            End Get
            Set(value As String)
                m_Id = value
            End Set
        End Property

        Public Property Password As String
            Get
                Return m_Password
            End Get
            Set(value As String)
                m_Password = value
            End Set

        End Property


        Public ReadOnly Property Id As String Implements IUser(Of String).Id
            Get
                Return m_Id
            End Get
        End Property

        Public Property UserName As String Implements IUser(Of String).UserName
    End Class

    Public Class AppUserStore
        Implements IUserStore(Of AppUser)
        Implements IUserPasswordStore(Of AppUser)

        Private Shared users As New List(Of AppUser)() From { _
            New AppUser() With { _
                .UserId = "user1", _
                .UserName = "user1", _
                .Password = "user1"
            } _
        }

        Public Function CreateAsync(ByVal user As AppUser) As Task Implements IUserStore(Of AppUser).CreateAsync
            users.Add(user)
            Return Task.Delay(0)
        End Function

        Public Async Function DeleteAsync(ByVal user As AppUser) As Task Implements IUserStore(Of AppUser).DeleteAsync
            Dim target = Await Me.FindByIdAsync(user.Id)
            If target Is Nothing Then
                Return
            End If

            users.Remove(target)
        End Function

        Public Function FindByIdAsync(userId As String) As Task(Of AppUser) Implements IUserStore(Of AppUser).FindByIdAsync
            Return Task.FromResult(users.FirstOrDefault(Function(u) u.Id = userId))
        End Function

        Public Function FindByNameAsync(userName As String) As Task(Of AppUser) Implements IUserStore(Of AppUser).FindByNameAsync
            Return Task.FromResult(users.FirstOrDefault(Function(u) u.UserName = userName))
        End Function

        Public Async Function UpdateAsync(user As AppUser) As Task Implements IUserStore(Of AppUser).UpdateAsync
            Dim target = Await Me.FindByIdAsync(user.Id)
            If target Is Nothing Then
                Return
            End If

            target.UserName = user.UserName
        End Function

        Public Sub Dispose() Implements IUserStore(Of AppUser).Dispose
        End Sub

        Public Function GetPasswordHashAsync(user As AppUser) As Task(Of String) Implements IUserPasswordStore(Of AppUser).GetPasswordHashAsync
            Return Task.FromResult(New PasswordHasher().HashPassword(user.Password))
        End Function

        Public Function HasPasswordAsync(user As AppUser) As Task(Of Boolean) Implements IUserPasswordStore(Of AppUser).HasPasswordAsync
            Return Task.FromResult(True)
        End Function

        Public Function SetPasswordHashAsync(user As AppUser, passwordHash As String) As Task Implements IUserPasswordStore(Of AppUser).SetPasswordHashAsync
            Return Task.Delay(0)
        End Function


    End Class


End Namespace